/* generic svp rendering function and callback callback */

/*

  predefined MACRO:

  ROP_CALLBACK_DECL : additional declaration and init code required if needed
  ROP_CALLBACK_DRAW(x,y,l,v) : the rastering code

 */
#define MAKE_ROP_CALLBACK(function_name, private_data) \
void function_name(void *callback_data, int y, int start, ArtSVPRenderAAStep *steps, int n_steps) { \
  int run_x0, run_x1; \
  art_u32 running_sum = start; \
  int x0, x1; \
  int k; \
  private_data * data=(private_data*)callback_data; \
  ROP_CALLBACK_DECL \
  x0   = data->x0; \
  x1   = data->x1; \
  if (n_steps > 0) { \
    run_x1 = steps[0].x; \
    if (run_x1 > x0) { ROP_CALLBACK_DRAW(x0, y, run_x1 - x0,running_sum >> 16) } \
    for (k = 0; k < n_steps - 1; k++) { \
      running_sum += steps[k].delta; \
      run_x0 = run_x1; \
      run_x1 = steps[k + 1].x; \
      if (run_x1 > run_x0) { ROP_CALLBACK_DRAW(run_x0, y, run_x1 - run_x0,running_sum >> 16) } \
    } \
    running_sum += steps[k].delta; \
    if (x1 > run_x1) { ROP_CALLBACK_DRAW(run_x1, y, x1 - run_x1,running_sum >> 16) } \
  } \
  else { ROP_CALLBACK_DRAW(x0, y, x1 - x0, running_sum >> 16) } \
} \

/*

  predifined MACRO:
  
  SVP_ROP_ARGS : additionnal args starting with a ','
  SVP_ROP_DECL : additionnal declarations
  SVP_ROP_INIT : additionnal init code
  SVP_ROP_EXIT : additionnal cleanup code

 */

#define MAKE_SVP_ROP(function_name,callback_name, private_data) \
void function_name(ggi_visual_t vis, const ggiart_svp_t svp, const \
		   ggiart_uta_t uta SVP_ROP_ARGS) \
{ \
  ggi_mode mode; \
  int w,h,y,x; \
  ArtUtaBbox ub; \
  private_data data; \
  SVP_ROP_DECL \
  ggiGetMode(vis,&mode); \
  data.x0=0; \
  data.x1=w=mode.virt.x; \
  h=mode.virt.y; \
  data.vis=vis; \
  SVP_ROP_INIT \
  if(!uta) { art_svp_render_aa(svp, 0, 0, w, h, callback_name, &data); \
  } \
  else { \
    w=uta->width; \
    h=uta->height; \
    for (y = 0; y < h; y++) \
      for (x = 0; x < w; x++) { \
	ub = uta->utiles[y * w + x]; \
	if (ub) { \
	  data.x0=(uta->x0 + x) * ART_UTILE_SIZE + ART_UTA_BBOX_X0(ub); \
	  data.x1=(uta->x0 + x) * ART_UTILE_SIZE + ART_UTA_BBOX_X1(ub); \
	  art_svp_render_aa(svp, \
			    data.x0, \
			    (uta->y0 + y) * ART_UTILE_SIZE + ART_UTA_BBOX_Y0(ub), \
			    data.x1+1, \
			    (uta->y0 + y) * ART_UTILE_SIZE + ART_UTA_BBOX_Y1(ub), \
			    callback_name, \
			    &data); \
	} \
      } \
  } \
  SVP_ROP_EXIT \
} \


